/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package info.petar.texinline;

import java.lang.reflect.Method;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Reader;

import java.util.Iterator;
import java.util.LinkedList;

import xtc.tree.LineMarker;
import xtc.tree.Node;
import xtc.tree.Node;
import xtc.tree.Pragma;
import xtc.tree.Printer;
import xtc.tree.SourceIdentity;
import xtc.tree.Visitor;

import xtc.parser.Result;
import xtc.parser.SemanticValue;
import xtc.parser.ParseError;

import info.petar.texinline.Constants;
import info.petar.texinline.LatexRenderer;
import info.petar.texinline.HtmlGenerator;
import info.petar.texinline.TexInlineParser;


public final class CompilerMain {

  private static String[] files;
  private static boolean error = false;

  private CompilerMain () { }
  
  //================================================= printParseError  
  private static void printParseError (ParseError err, TexInlineParser parser) 
    throws IOException {
    
    System.err.println ();
    System.err.println (err.msg + " at " + err.index);
    error = true;
  }

  //====================================================== printUsage  
  private static void printUsage () {
    System.err.println ("Usage: ticc [Options] <FileName>+\n");
    System.err.println ("    --show-latex-output/--lo   " +
                        "Show LaTeX output (Default: Hide LaTeX output)");
    System.err.println ("    --show-latex-source/--ls   " +
                        "Show LaTeX source (Default: Hide LaTeX source)");
    System.err.println ("    --show-dvipng-output/--do  " +
                        "Show DVIPNG output (Default: Hide DVIPNG output)");
    System.err.println ("    --latex LatexCmd           " +
                        "Set LaTeX command (Default: latex)");
    System.err.println ("                               " +
                        "You can include LaTeX options");
    System.err.println ("    --dvipng DviPngCmd         " +
                        "Set DVIPNG command (Default: dvipng)");
    System.err.println ("                               " +
                        "You can include DVIPNG options");
    System.err.println ("    --dpi Resolution           " +
                        "Set image resolution (Default: 115)");
    //System.err.println ("    --no-anti-alias/--naa      " +
    //"Do not use anti-alias (Default: Use anti-alias)");
    System.err.println ("    --bg Background            " +
                        "Set background (Default: Transparent)");
    System.err.println ("                               " +
                        "TeX-style color or 'Transparent'\n");
    
    System.exit (1);
  }
  
  private static void parseOpts (String args[]) {
    LinkedList<String> f = new LinkedList ();
    if ((null == args) || (0 == args.length))
      printUsage ();
    
    for (int a = 0; a < args.length; a++)
      if (args[a].equals ("--show-latex-output") ||
          args[a].equals ("--lo"))
        LatexRenderer.optShowLatexOutput = true;
      else if (args[a].equals ("--show-latex-source") ||
          args[a].equals ("--ls"))
        LatexRenderer.optShowLatexSource = true;
      else if (args[a].equals ("--show-dvipng-output") ||
          args[a].equals ("--do"))
        LatexRenderer.optShowDvipngOutput = true;
      else if (args[a].equals ("--latex")) {
        a++;
        if (a >= args.length)
          printUsage ();
        else
          LatexRenderer.optLatexCmd = args[a];
      } else if (args[a].equals ("--dvipng")) {
        a++;
        if (a >= args.length)
          printUsage ();
        else
          LatexRenderer.optDvipngCmd = args[a];
      } else if (args[a].equals ("--dpi")) {
        a++;
        if (a >= args.length)
          printUsage ();
        else
          LatexRenderer.optDpi = args[a];
      } else if (args[a].equals ("--no-anti-alias") ||
                 args[a].equals ("--naa"))
        LatexRenderer.optNoAntiAlias = true;
      else if (args[a].equals ("--bg")) {
        a++;
        if (a >= args.length)
          printUsage ();
        else
          LatexRenderer.optBg = args[a];
      } else
        f.add (args[a]);

    if (f.size () > 0) {
      files = new String[f.size ()];
      for (int i = 0; i < files.length; i++)
        files [i] = f.poll ();
    } else
      printUsage ();
  }

  //========================================================= process
  private static void process (String file) {
    Reader in = null;
    try {
      try {
        in = new BufferedReader (new FileReader (file));
      } catch (Exception ex) {
        System.err.println ("Cannot open file \"" + file + "\"!");
        return ;
      }
      TexInlineParser p  = 
        new TexInlineParser (in, file, (int) new File (file).length ());
      Result r = p.pSpecification (0);
      
      if (r.hasValue ()) {
        SemanticValue v = (SemanticValue) r;

        Node root = (Node) v.value;
        /*
          Printer printer = 
          new Printer 
          (new BufferedWriter 
          (new OutputStreamWriter (System.out)));
        

          if (v.value instanceof Node) {
          Printer raw_printer = 
          new Printer 
          (new BufferedWriter 
          (new OutputStreamWriter (System.err)));
          
          raw_printer.pln ().format (v.value).pln ().pln ().flush ();
          } else {
          System.err.println (v.value.toString ());
          }
        */
          
        // do generation
        
        HtmlGenerator hg = new HtmlGenerator (file, root);
        hg.generate ();
        
      } else {
        ParseError err = (ParseError) r;
        printParseError (err, p);
      }
      
    } catch (Exception ex) {
      System.err.println ("===> This is techincal:");
      ex.printStackTrace ();
      System.exit (1);
    } finally {
      try {
        in.close ();
      } catch (Throwable th) { }
    }
  }
  
  //============================================================ main
  public static void main (String[] args) {

    System.err.print (Constants.NAME + " ");
    System.err.println (Constants.VERSION + 
                        " (See " + 
                        Constants.URL+ 
                        ")");
    System.err.println (Constants.COPY);
    System.err.println (Constants.SEP);
    
    parseOpts (args);

    for (int i = 0; i < files.length; i++) {
      System.err.println("Processing " + files[i] + " ...");
      try {
        process (files[i]);
      } catch (Throwable x) {
        error = true;
        while (null != x.getCause())
          x = x.getCause();
        if (x instanceof FileNotFoundException)
          System.err.println (x.getMessage());
        else
          x.printStackTrace ();
      }
    }
    
    if (error)
      System.exit (1);
    else
      System.exit (0);
  }

} // CompilerMain
