/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package info.petar.texinline;

import java.io.*;
import java.util.*;
import xtc.tree.Attribute;
import xtc.tree.AttributeList;
import xtc.tree.Node;
import xtc.util.Utilities;
import info.petar.texinline.LatexRenderer;


public final class HtmlGenerator {
  
  private String htmltex_path;
  private String html_basedir;
  private String html_filename;
  private File pngdir;
  private File tmpdir;

  private Node ast;

  public Node getAst () {
    return ast;
  }

  public HtmlGenerator (String htmltex_path, Node ast) {
    this.htmltex_path = htmltex_path;
    processPath ();
    this.ast = ast;
  }
  
  //================================================================
  
  private void processPath () {
    File path = new File (htmltex_path);
    html_basedir = path.getParent ();
    if (html_basedir == null)
      html_basedir = ".";
    html_filename = path.getName ();
    String[] parts = html_filename.split ("\\.");
    html_filename = parts [0];

    pngdir = new File (html_basedir + "/.texinline");
    tmpdir = new File (html_basedir + "/.texinline.tmp");
  }

  private void mkPngDir () throws Exception {
    if (pngdir.exists ())
      return ;
    if (!pngdir.mkdir ()) {
      System.err.println ("Cannot create directory: " + pngdir);
      throw new Exception ();
    }
  }

  private void mkTmpDir () throws Exception {
    if (tmpdir.exists ())
      return ;
    if (!tmpdir.mkdir ()) {
      System.err.println ("Cannot create directory: " + tmpdir);
      throw new Exception ();
    }
  }
  
  private static boolean rmDir (File dir) {
    if (dir.isFile ())
      return dir.delete ();
    else {
      File[] ls = dir.listFiles ();
      for (int i = 0; i < ls.length; i++)
        if (!rmDir (ls[i]))
          return false;
      return dir.delete ();
    }
  }

  private void rmTmpDir () throws Exception {
    if (!rmDir (tmpdir)) {
      System.err.println ("Cannot remove directory: " + tmpdir);
      // throw new Exception ();
    }
  }

  public void generate () throws Exception {
    // make directories
    mkPngDir ();
    mkTmpDir ();
    
    // prepare header
    StringBuilder hdr_bldr = new StringBuilder ();
    for (Object o1 : ast) {
      Node item = (Node) o1;
      if (item.get (0) instanceof HeadElement) {
        // System.err.println ("HEAD element encountered I");
        for (Object o2 : ((HeadElement) item.get (0)).getInsides ())
          if (o2 instanceof LatexElement) {
            // System.err.println ("LATEX element encountered I");
            hdr_bldr.append (((LatexElement) o2).getText ());
          }
      }
    }
    String hdr = 
      "\\nonstopmode\n" +
      "\\documentclass[12pt]{article}\n" +
      "\\pagestyle{empty}" +
      hdr_bldr.toString ();
    
    // create html file
    File htmlfile = new File (html_basedir + "/" + html_filename + ".html");
    FileWriter htmlwriter = new FileWriter (htmlfile);

    // traverse ast
    LatexRenderer renderer 
      = new LatexRenderer (tmpdir.toString (), pngdir.toString ());
    for (Object o1 : ast) {
      Object elm = ((Node) o1).get (0);
      if (elm instanceof String)
        htmlwriter.write ((String) elm);
      else if (elm instanceof HeadElement) {
        // System.err.println ("HEAD element encountered II");
        htmlwriter.write ("<head>");
        
        htmlwriter.write 
          ("<!--\n" +
           "This file was automatically generated with:\n" +
           Constants.NAME + " " + Constants.VERSION + 
           " (See " + Constants.URL + ")\n" + 
           Constants.COPY + "\n" + 
           "On " + (new Date ()).toString () + "\n" +
           "-->\n");

        htmlwriter.write ("<style><!-- " +
                          "img.latex { vertical-align: middle; }" +
                          "--></style>");

        for (Object o2 : ((HeadElement) elm).getInsides ())
          if (o2 instanceof String)
            htmlwriter.write (((String) o2) + " ");
        htmlwriter.write ("</head>");
      } else if (elm instanceof LatexElement) {
        // System.err.println ("LATEX element encountered II");
        
        // latex element
        String inline = ((LatexElement) elm).getText ();
        if (!inline.trim ().equals ("")) {
          String latex = 
            "\\begin{document}" + 
            inline +
            "\\end{document}\n";
          
          // image annotation logic
          String unescaped_alt;
          String elm_alt = ((LatexElement) elm).getAttrib ("alt");
          if (elm_alt != null)
            unescaped_alt = elm_alt;
          else
            unescaped_alt = inline;
          
          String imgfile = 
            renderer.latexToPng (((LatexElement) elm).getTag (), 
                                 hdr + latex);
          System.out.println ("---> Evaluating <latex>...</latex>: " + inline);
          htmlwriter.write ("<img class='latex' src=\"" +
                            imgfile +
                            "\" alt=\"" +
                            Sgml.escapeDouble (unescaped_alt) +
                            "\"/>");
        }
        
      } else 
        assert false;
    }

    // close html file
    htmlwriter.flush ();
    htmlwriter.close ();
    
    // remove tmp dir
    rmTmpDir ();
  }

} // HtmlGenerator
