/*
 * TexInline
 * Copyright (C) 2007 Petar Maymounkov (petar@csail.mit.edu)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package info.petar.texinline;

import java.io.*;
import java.security.MessageDigest;
import info.petar.texinline.Tag;
import info.petar.texinline.Armor;


public final class LatexRenderer {

  public static boolean optShowLatexOutput = false;
  public static boolean optShowLatexSource = false;
  public static boolean optShowDvipngOutput = false;
  public static String optLatexCmd = "latex";
  public static String optDvipngCmd = "dvipng";
  public static String optDpi = "115";
  public static String optGamma = "1.0";
  public static boolean optNoAntiAlias = false;;
  public static String optBg = "Transparent";

  
  //====================================================================
  private File tmpdir;
  private File pngdir;
  private MessageDigest md;

  public LatexRenderer (String tmpdir, String pngdir) throws Exception {
    this.tmpdir = new File (tmpdir);
    this.pngdir = new File (pngdir);
    md = MessageDigest.getInstance ("SHA");
  }

  private String calcHash (String latex) throws Exception {
    byte[] lbb = latex.getBytes ();
    md.reset ();
    md.update (lbb);
    byte[] hbb = md.digest ();
    md.reset ();
    return new String (Armor.armor32 (hbb));
  }

  private boolean cmdLatex (String texfile) throws Exception {

    String cmd = optLatexCmd + " " + texfile;
    if (optShowLatexOutput)
      System.out.println ("==> Running LaTeX: " + cmd);
    Process p = Runtime.getRuntime ().
      exec (cmd, null, tmpdir);
    
    if (optShowLatexOutput) {
      System.out.println ("==> LaTeX output:");
      String line;
      BufferedReader input =
        new BufferedReader
        (new InputStreamReader (p.getInputStream ()));
      while ((line = input.readLine ()) != null)
        System.out.println(line);
      input.close ();
    }
    
    p.waitFor ();
    return p.exitValue () == 0;
  }
  
  private boolean cmdDviPng (String hash, 
                             String effDpi, 
                             String effGamma,
                             String effBg) throws Exception {
    
    if (effDpi == null)
      effDpi = optDpi;
    if (effGamma == null)
      effGamma = optGamma;
    if (effBg == null)
      effBg = optBg;

    String cmd = optDvipngCmd +
      " " + 
      "-T tight " +
      "-gamma " + effGamma + " " +
      "-D " + effDpi + " " +
      "--png " +
      "-bg " + effBg + " " +
      "-o " + 
      hash + ".png " +
      hash + ".dvi";
    
    if (optShowDvipngOutput)
      System.out.println ("==> Running DVIPNG: " + cmd);
    Process p = Runtime.getRuntime ().
      exec (cmd, null, tmpdir);
    
    if (optShowDvipngOutput) {
      System.out.println ("==> DVIPNG output:");
      String line;
      BufferedReader input =
        new BufferedReader
        (new InputStreamReader (p.getInputStream ()));
      while ((line = input.readLine ()) != null)
        System.out.println(line);
      input.close ();
    }
    
    p.waitFor ();
    return p.exitValue () == 0;
  }

  public String latexToPng (Tag tag, String latex) throws Exception {
    String hash = tag.getAttrib ("file");
    if (hash == null)
      hash = calcHash (latex);
    
    // create latex source
    String texfile = tmpdir.toString () + "/" + hash + ".tex";
    FileWriter latex_writer = new FileWriter (texfile);
    latex_writer.write (latex);
    latex_writer.flush ();
    latex_writer.close ();
    
    // execute latex
    if (optShowLatexSource) {
      System.out.println ("==> LaTeX source:");
      System.out.println (latex);
    }
    if (!cmdLatex (hash + ".tex")) {
      System.err.println ("LaTeX error in:\n" + latex + "\n");
      throw new Exception ();
    }
    
    // execute dvipng
    
    if (!cmdDviPng (hash, 
                    tag.getAttrib ("dpi"),
                    tag.getAttrib ("gamma"),
                    tag.getAttrib ("bg"))) {
      System.err.println ("DviPng error for:\n" + latex + "\n");
      throw new Exception ();
    }
    
    // move png file
    File pngfile = new File (tmpdir.toString () + "/" + hash + ".png");
    File pngfile2 = new File (pngdir.toString () + "/" + hash + ".png");
    if (pngfile2.exists ())
      if (!pngfile2.delete ()) {
        System.err.println ("Destination file \"" + 
                            pngfile2.toString () +
                            "\" already exists and cannot be removed!");
        throw new Exception ();
      }
    if (!pngfile.renameTo (pngfile2)) {
      System.err.println ("Cannot move \"" +
                          pngfile.toString () +
                          "\" to \"" +
                          pngfile2.toString () +
                          "\"!");
      throw new Exception ();
    }
    
    return ".texinline/" + hash + ".png";
  }

} // LatexRenderer
